import { Button } from '@graphpolaris/shared/lib/components/buttons';
import { Panel, Resizable } from '@graphpolaris/shared/lib/components/layout';
import {
  useAppDispatch,
  useAuthorizationCache,
  useML,
  useQuerybuilderGraph,
  useQuerybuilderSettings,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { EventBus } from '@graphpolaris/shared/lib/data-access/api/eventBus';
import { URLParams, setParam } from '@graphpolaris/shared/lib/data-access/api/url';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { wsQueryRequest } from '@graphpolaris/shared/lib/data-access/broker';
import { queryingBackend, resetGraphQueryResults } from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryBuilder, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { SideNavTab, Sidebar } from '@graphpolaris/shared/lib/sidebar';
import { SearchBar } from '@graphpolaris/shared/lib/sidebar/search/SearchBar';
import { VisualizationPanel } from '@graphpolaris/shared/lib/vis';
import { ConfigPanel } from '@graphpolaris/shared/lib/vis/components/config';
import { VisualizationManager } from '@graphpolaris/shared/lib/vis/manager';
import { useEffect, useState } from 'react';
import { Navbar } from '../components/navbar/navbar';
import { Onboarding } from '../components/onboarding/onboarding';

import { CalendarVis } from '@graphpolaris/shared/lib/vis/visualizations/calendar/calendarvis';

import { KeyboardArrowDown, KeyboardArrowRight } from '@mui/icons-material';

export type App = {
  load?: string;
};

export function App(props: App) {
  const auth = useAuthorizationCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const ml = useML();
  const session = useSessionCache();
  const dispatch = useAppDispatch();
  const queryBuilderSettings = useQuerybuilderSettings();
  const manager = VisualizationManager();

  const [authCheck, setAuthCheck] = useState(false);
  const [tab, setTab] = useState<SideNavTab>('Schema');
  const [visFullSize, setVisFullSize] = useState<boolean>(false);
  const [expanded, setExpanded] = useState<boolean>(true);

  const runQuery = () => {
    if (session?.currentSaveState && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        dispatch(queryingBackend());
        wsQueryRequest(Query2BackendQuery(session.currentSaveState, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    if (props.load) {
      setParam(URLParams.saveState, props.load);
    }
  }, [props]);

  return (
    <div className="h-screen w-screen overflow-clip">
      <EventBus
        onRunQuery={runQuery}
        onAuthorized={() => {
          setAuthCheck(true);
        }}
      />
      {authCheck && (
        <>
          <Onboarding />
          <DashboardAlerts />
          <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
            <div className="flex flex-col h-screen max-h-screen relative">
              <aside className="absolute w-full h-12">
                <Navbar />
              </aside>
              <main className="grow flex flex-row h-screen pt-12">
                <Sidebar onTab={(tab) => setTab(tab)} tab={tab} />
                <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
                  {tab !== undefined ? (
                    <div className="flex flex-col w-full h-full">
                      {tab === 'Search' && <SearchBar onRemove={() => setTab(undefined)} />}
                      {tab === 'Schema' && <Schema auth={authCheck} onRemove={() => setTab(undefined)} />}
                    </div>
                  ) : null}

                  <div className="w-full h-full flex flex-col justify-between">
                    <Panel title="Panel" tooltips={undefined}>
                      <VisualizationPanel
                        manager={manager}
                        fullSize={() => {
                          setVisFullSize(!visFullSize);
                          tab === undefined && setTab('Schema');
                          tab !== undefined && setTab(undefined);
                        }}
                      />
                    </Panel>
                    <>
                      <div
                        className="flex w-full py-0 px-2 bg-secondary-50 cursor-pointer border-y items-center gap-1"
                        onClick={() => setExpanded(!expanded)}
                      >
                        <Button
                          size="xs"
                          variant="ghost"
                          iconComponent={expanded ? <KeyboardArrowDown /> : <KeyboardArrowRight />}
                          onClick={() => setExpanded(!expanded)}
                        />
                        <span className="text-xs font-semibold text-secondary-600 truncate">Filter</span>
                      </div>
                      {expanded && (
                        <div className="h-[36%] pt-2 pl-2">
                          <CalendarVis />
                        </div>
                      )}
                    </>

                    <QueryBuilder onRunQuery={runQuery} />
                  </div>
                </Resizable>
                <div className="info-panel flex h-full w-60 ml-[3px] shrink-0 overflow-auto bg-light border">
                  <ConfigPanel manager={manager} />
                </div>
              </main>
            </div>
          </div>
        </>
      )}
    </div>
  );
}

export default App;
