import { describe, expect, it } from 'vitest';
import { getDimension, isSpatial } from './dimensions';

describe('getDimension', () => {
  it('should return "categorical" for an empty array', () => {
    expect(getDimension([])).toBe('categorical');
  });

  it('should return the correct temporal dimension for a single value', () => {
    expect(getDimension('2022-01-01')).toBe('temporal');
    expect(getDimension('2019-11-01T10:30:00.000Z')).toBe('temporal');
    expect(getDimension('2022-01')).toBe('temporal'); // Year and month
    expect(getDimension('2022-01-01')).toBe('temporal'); // Full date
    expect(getDimension('2022-01-01T10:30')).toBe('temporal'); // Date and time without seconds
    expect(getDimension('2022-01-01T10:30:00')).toBe('temporal'); // Date and time with seconds
    expect(getDimension('2022-01-01T10:30:00.000Z')).toBe('temporal'); // Date and time with milliseconds
    expect(getDimension(new Date().toDateString())).toBe('temporal'); // Week date with day
  });

  it('unfortunate error cases no solution for date.getTime() being a valid date number', () => {
    expect(getDimension(new Date().getTime())).toBe('numerical'); // Week date with day
    expect(getDimension(new Date().getTime() - 1000000)).toBe('numerical'); // Week date with day
  });

  // TODO fix this
  // it('should return the correct spatial dimension for partial Dutch addresses', () => {
  //   expect(getDimension('1011 AC')).toBe('spatial'); // Only postcode
  //   expect(getDimension('Prins Hendrikkade 85')).toBe('spatial'); // Only street
  //   expect(getDimension('Amsterdam')).toBe('spatial'); // Only city
  //   expect(getDimension('2511 CV, Den Haag')).toBe('spatial'); // Postcode and city
  //   expect(getDimension('Buitenhof 39, Den Haag')).toBe('spatial'); // Street and city
  //   expect(getDimension('2511 CV, Buitenhof 39')).toBe('spatial'); // Postcode and street
  // });

  it('should return the correct dimension for a single value', () => {
    expect(getDimension('2022-01-01')).toBe('temporal');
    expect(getDimension('2019-11-01T10:30:00.000Z')).toBe('temporal');
    expect(getDimension({ lat: 40, lon: -75 })).toBe('spatial');
    expect(getDimension(42)).toBe('numerical');
    expect(getDimension('apple')).toBe('categorical');
  });

  it('should return the correct dimension for a single value', () => {
    expect(getDimension('2022-01-01')).toBe('temporal');
    expect(getDimension('2019-11-01T10:30:00.000Z')).toBe('temporal');
    expect(getDimension({ lat: 40, lon: -75 })).toBe('spatial');
    expect(getDimension(42)).toBe('numerical');
    expect(getDimension('apple')).toBe('categorical');
  });

  it('should return the correct dimension for an array of values', () => {
    expect(getDimension(['2022-01-01', '2022-01-02'])).toBe('temporal');
    expect(getDimension({ lat: 40, lon: -75 })).toBe('spatial');
    expect(getDimension([42])).toBe('numerical');
    expect(getDimension([42, 43])).toBe('numerical');
    expect(getDimension(['apple', 'banana'])).toBe('categorical');
  });
});

describe('SAFParsing', () => {
  it('should return the correct dimension for all SAF C4C data', () => {
    // expect(getDimension('0')).toBe('numerical');
    expect(getDimension('E01VERMOEDEN')).toBe('categorical');
    // expect(getDimension('3021 JB')).toBe('spatial');
    expect(getDimension('false')).toBe('categorical');
    expect(getDimension('2019-01-30T18:45:00.000Z')).toBe('temporal');
    expect(getDimension('2019278157')).toBe('numerical');
    // expect(getDimension('Claes de Vrieselaan')).toBe('spatial');
    expect(
      getDimension(
        'Wij hebben een schade van 30 januari 2019 in behandeling waarbij het voertuig van onze verzekerde (03FGGB) bij betrokken is geweest. In tegenstelling tot wat op het schadeformulier staat is onze verzekerde: V. Jitan, Reitdiep 61, 2901 RD Capelle aan den Ijssel en niet JMR Koeriersdiensten. Wij vermoeden dat er sprake is van een opzetaanrijding. Eerder is Ramdien (ook een vennoot van JMR Koeriersdiensten) negatief betrokken geweest bij een scene aanrijding. Inshared doet ook onderzoek in een andere zaak waar onze auto bij betrokken is en ik heb vernomen dat zij geen contact kunnen krijgen met verzekerde. Op het adres zou inmiddels een vrouw wonen die gek wordt van de herinneringen, deurwaarders etc. Ook de heer Falix zou geen gehoor geven.',
      ),
    ).toBe('categorical');
    // expect(getDimension('Rotterdam')).toBe('spatial');
  });

  // AantalSAF: '0';
  // AndereVoertuigenBetrokken: 'false';
  // Fraude: 'E01VERMOEDEN';
  // GeschatSchadeBedrag: '0';
  // Gewonden: 'false';
  // Postcode: '3021 JB';
  // Schadedatum: '2019-01-30T18:45:00.000Z';
  // Schadenummer: '2019278157';
  // Straatnaam: 'Claes de Vrieselaan';
  // Toedracht: "Wij hebben een schade van 30 januari 2019 in behandeling waarbij het voertuig van onze verzekerde (03FGGB) bij betrokken is geweest. In tegenstelling tot wat op het schadeformulier staat is onze verzekerde: V. Jitan, Reitdiep 61, 2901 RD Capelle aan den Ijssel en niet JMR Koeriersdiensten. Wij vermoeden dat er sprake is van een opzetaanrijding. Eerder is Ramdien (ook een vennoot van JMR Koeriersdiensten) negatief betrokken geweest bij een scene aanrijding. Inshared doet ook onderzoek in een andere zaak waar 'onze' auto bij betrokken is en ik heb vernomen dat zij geen contact kunnen krijgen met verzekerde. Op het adres zou inmiddels een vrouw wonen die gek wordt van de herinneringen, deurwaarders etc. Ook de heer Falix zou geen gehoor geven.";
  // VbVAanrijdingnummer: '2';
  // VerzekerdVoertuigKm: '0';
  // Woonplaats: 'Rotterdam';
});
