import { DimensionType } from '@graphpolaris/shared/lib/schema';
import moment from 'moment';

export const isNumerical = (value: any): boolean => {
  if (typeof value === 'number') return true;
  const parsedNumber = +value;
  if (!isNaN(parsedNumber)) return true;
  return false;
};

export const isSpatial = (value: any): boolean => {
  if (Array.isArray(value) && value.length === 2) {
    const [latitude, longitude] = value;
    if (typeof latitude === 'number' && typeof longitude === 'number') {
      return true;
    }
  } else if (typeof value === 'object' && value !== null) {
    if ('latitude' in value && 'longitude' in value) {
      const { latitude, longitude } = value;
      if (typeof latitude === 'number' && typeof longitude === 'number') {
        return true;
      }
    }
  } else if (typeof value === 'string') {
    const regex = /^-?\d+(.\d+)?, -?\d+(.\d+)?$/;
    return regex.test(value.toString().trim());
  }

  const hasLetters = /[a-zA-Z]/.test(value);
  const hasNumbers = /\d/.test(value);
  return hasLetters && hasNumbers;
};

/**
 * Checks if a given value is temporal.
 *
 * Is a stepwise check with gradually
 * being more and more computational expensive.
 * This function uses the Moment.js library to parse the input value as a date or time.
 * If the parsing is successful, the function returns true, indicating that the value is temporal. If the parsing is not successful, the function tries to parse the value using the built-in Date.parse method. If this parsing is successful, the function also returns true. If both parsing attempts fail, the function checks if the value does not match a regular expression that tests for a number (possibly with a decimal point). If the value does not match this regular expression, the function returns true, indicating that the value is temporal. If all checks fail, the function returns false, indicating that the value is not temporal.
 *
 * @param {any} value - The value to check.
 * @returns {boolean} - Returns true if the value is temporal, false otherwise.
 */
export function isTemporal(value: any): boolean {
  const datetime = moment(value);
  // console.log(
  //   value,
  //   'datetime.isValid()',
  //   datetime.isValid(),
  //   'Date.parse(value)',
  //   Date.parse(value),
  //   '!isNaN(Date.parse(value)',
  //   !isNaN(Date.parse(value)),
  //   'regex.test(value)',
  //   /^[0-9]+[.]{0,1}[0-9]*$/.test(value),
  // );
  if (typeof value === 'number') {
    return false;
  } else if (datetime.isValid() && datetime.toArray().indexOf(Number.NaN) === -1) {
    return true;
  }

  // else if (!isNaN(Date.parse(value))) {
  //   return true;
  // }
  return false;
}

export const getDimension = (value: any): DimensionType => {
  if (Array.isArray(value)) {
    if (value.length === 0) return 'categorical';

    type DimensionCount = {
      DimensionType: number;
    };

    /**
     * TODO extract the most occurring dimension from array and return it as the dimension
     */
    // const dimensions = value.map((v) => getDimension(v));
    // const mostOccurring = dimensions.reduce((acc: DimensionCount, curr: DimensionType) => {
    //   acc[curr] = (acc[curr] || 0) + 1;
    //   return acc;
    // }, {});

    // const entries = Object.entries(mostOccurring);
    // const highestEntry = entries.reduce((a: DimensionCount, b: DimensionCount) => (a[1] > b[1] ? a : b));
    // return highestEntry[0];

    return getDimension(value[0]);
  }
  if (isObject(value)) {
    const keys = Object.keys(value);
    if (keys.length > 0) {
      if (
        (keys.length === 2 && keys.includes('latitude') && keys.includes('longitude')) ||
        (keys.length === 2 && keys.includes('lat') && keys.includes('lon'))
      ) {
        return 'spatial';
      }
      const firstKey = keys[0];
      return getDimension(value[firstKey]);
    }
  }

  const numerical = isNumerical(value);
  const temporal = isTemporal(value);
  if (numerical && !temporal) {
    return 'numerical';
  } else if (temporal) {
    return 'temporal';
  } else if (isSpatial(value)) {
    return 'spatial';
  }
  return 'categorical';
};

function isObject(value: any) {
  return value && typeof value === 'object' && value.constructor === Object;
}
