import { DateAggregate } from './calendarvis.const';
import { DataStats, DateValue } from './calendarvis.types';
import { getWeekNumber } from './calendarvis.utils';

export const months = [
  'January',
  'February',
  'March',
  'April',
  'May',
  'June',
  'July',
  'August',
  'September',
  'October',
  'November',
  'December',
];
// enum Month {
//   January = 'January',
//   February = 'February',
//   March = 'March',
//   April = 'April',
//   May = 'May',
//   June = 'June',
//   July = 'July',
//   August = 'August',
//   September = 'September',
//   October = 'October',
//   November = 'November',
//   December = 'December',
// }

export class CalendarVisController {
  model!: DateValue[];
  stats!: DataStats;
  dateAggregate!: DateAggregate;
  showDayLabel!: boolean;
  showMonthLabel!: boolean;
  showYearLabel!: boolean;
  showWeekLabel!: boolean;

  constructor() {
    this.reset();
  }

  reset() {
    this.setModel([]);
    this.setStats({ min: 0, max: 0, avg: 0 });
    this.setDateAggregate(DateAggregate.days);
    this.setShowDayLabel(true);
    this.setShowWeekLabel(true);
    this.setShowMonthLabel(true);
    this.setShowYearLabel(true);
  }

  setModel(model: DateValue[]) {
    this.model = model;
    if (model.length === 0) {
      // console.error('setModel', 'no model');
      return;
    } else {
      this.setDateAggregate(this.dateAggregate);
    }
  }

  getModel() {
    return this.model;
  }

  getYears() {
    const years = this.model.reduce((acc: number[], curr: DateValue) => {
      const year = new Date(curr.date).getFullYear();
      if (!acc.includes(year)) {
        acc.push(year);
      }
      return acc;
    }, []);

    return years;
  }

  getModelItemsInYear(year: number) {
    const model: { [key: string]: DateValue[] } = {};

    const data = this.model.slice();
    // while input is not empty
    while (data.length > 0) {
      const item = data.shift();

      if (!item) break;
      const year = new Date(item.date).getFullYear().toString();
      if (!model[year]) {
        model[year] = [];
      }
      model[year].push(item);
    }

    return model[year.toString()];
  }

  getModelItemsInYearAndMonth(year: number, month: number) {
    const data = this.getModelItemsInYear(year);
    if (!data) return [];

    const filteredData = data.filter((item) => {
      const date = new Date(item.date);
      return date.getFullYear() === +year && date.getMonth() === month;
    });

    return filteredData;
  }

  getModelItemsInYearAndWeek(year: number, week: number) {
    const data = this.getModelItemsInYear(year);
    if (!data) return [];

    const filteredData = data.filter((item) => {
      const date = new Date(item.date);
      return date.getFullYear() === year && getWeekNumber(date) === week;
    });

    return filteredData;
  }

  getCollapsedWeeks() {
    return false;
  }

  setStats(arg0: DataStats) {
    this.stats = arg0;
  }

  setDateAggregate(dateAggregate: DateAggregate) {
    this.dateAggregate = dateAggregate;

    if (!this.model || this.model.length === 0) {
      // console.error('setDateAggregate', 'no model');
      return;
    }

    let min = Infinity;
    let max = -Infinity;
    let aggregate: DateValue[] = [];
    switch (dateAggregate) {
      case DateAggregate.days:
        // group model into days
        aggregate = this.model.reduce((acc: DateValue[], curr: DateValue) => {
          const date = new Date(curr.date);
          date.setHours(0, 0, 0, 0);
          const existingItem = acc.find((item) => item.date.getTime() === date.getTime());
          if (existingItem) {
            existingItem.value += curr.value;
          } else {
            acc.push({ date, value: curr.value });
          }
          return acc;
        }, []);
        break;
      case DateAggregate.weeks:
        break;
      case DateAggregate.months:
        break;
      case DateAggregate.years:
        // group model into days
        aggregate = this.model.reduce((acc: DateValue[], curr: DateValue) => {
          const date = new Date(curr.date);
          date.setHours(0, 0, 0, 0);
          date.setMonth(0, 1);

          const existingItem = acc.find((item) => item.date.getTime() === date.getTime());
          if (existingItem) {
            existingItem.value += curr.value;
          } else {
            acc.push({ date, value: curr.value });
          }
          return acc;
        }, []);

        break;
    }

    aggregate.forEach((aggregateItem) => {
      min = Math.min(min, aggregateItem.value);
      max = Math.max(max, aggregateItem.value);
    });
    // console.log('dayAggregate', dayAggregate, 'setDateAggregate', dateAggregate);
    this.setStats({ min, max, avg: (max - min) / 2 });
  }
  setShowDayLabel(arg0: boolean) {
    this.showDayLabel = arg0;
  }

  setShowWeekLabel(arg0: boolean) {
    this.showWeekLabel = arg0;
  }
  setShowMonthLabel(arg0: boolean) {
    this.showMonthLabel = arg0;
  }
  setShowYearLabel(arg0: boolean) {
    this.showYearLabel = arg0;
  }

  getStats(): DataStats {
    return this.stats;
  }
  getShowYearLabel() {
    return this.showYearLabel;
  }
  getAggregate() {
    return this.dateAggregate;
  }

  getShowMonthLabel() {
    return this.showMonthLabel;
  }

  getShowDayLabel() {
    return this.showDayLabel;
  }

  getShowWeekLabel() {
    return this.showWeekLabel;
  }
}
