import React, { useEffect, useMemo, useRef, useState } from 'react';
import { VISComponentType } from '../../common/types';
import { DateAggregate, TEMPORAL_ATTRIBUTES } from './calendarvis.const';
import { CalendarVisController } from './calendarvis.controller';
import './calendarvis.module.scss';
import TimelineTable from './components/TimelineTable';

import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { GraphMetadata } from '@graphpolaris/shared/lib/data-access/statistics';
import { useAppDispatch, useGraphQueryResult, useGraphQueryResultMeta } from '@graphpolaris/shared/lib/data-access/store/hooks';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import { DimensionType } from '../../..';

import { resultSetSelection, unSelect } from '@graphpolaris/shared/lib/data-access/store/interactionSlice';
import { getDatesForAttribute } from './calendarvis.utils';

import moment from 'moment';

export interface CalendarVisProps {}

export const CalendarVis = () => {
  const dispatch = useAppDispatch();

  const graphQueryResult = useGraphQueryResult();
  const meta = useGraphQueryResultMeta();
  const controller = useRef(new CalendarVisController());

  const [selectedDateAttribute, setSelectedDateAttribute] = useState('nodes.INCIDENT.Schadedatum');

  const dateAttributes = useMemo(() => determineDateAttributes('nodes', meta.nodes.types), [meta]);

  useEffect(() => {
    if (!graphQueryResult || !meta) return;
    // console.log('update Calendar useEffect', graphQueryResult, meta);
    controller.current.reset();

    if (!dateAttributes || dateAttributes.length === 0) return;

    let model = getDatesForAttribute(graphQueryResult, selectedDateAttribute);
    // let mmodel = getRandomDates(5000);

    model = model.sort((a, b) => a.date.getTime() - b.date.getTime());
    controller.current.setModel(model);
  }, [graphQueryResult, dateAttributes, selectedDateAttribute]);
  // const [model, setModel] = useState([] as DateValue[]);

  // useEffect(() => {

  const dateAggregate = DateAggregate.days;
  controller.current.setDateAggregate(dateAggregate);

  controller.current.setShowYearLabel(true);
  controller.current.setShowMonthLabel(true);
  controller.current.setShowWeekLabel(true);
  controller.current.setShowDayLabel(true);

  const handleMouseUp = (event: React.MouseEvent) => {
    const selection = window.getSelection();
    if (selection) {
      const range = selection.getRangeAt(0);
      const selectionContents = range.cloneContents();

      // Create a new DOM node to hold the cloned contents
      const tempDiv = document.createElement('div');
      tempDiv.appendChild(selectionContents);

      // Query all elements with data- attributes
      const dataElements = tempDiv.querySelectorAll('.day');

      if (!dataElements || dataElements.length === 0) {
        dispatch(unSelect());
      } else {
        const startDate = moment(dataElements[0].getAttribute('data-key')).toDate();
        const endDate = moment(dataElements[dataElements.length - 1].getAttribute('data-key')).toDate();

        // filter selection nodes
        const split = selectedDateAttribute.split('.');
        const selectionNodes = graphQueryResult.nodes.filter((node) => {
          const storedDate = node.attributes[split[2]];
          if (!storedDate) return false;
          const dateObj = moment(storedDate).toDate();

          return dateObj >= startDate && dateObj <= endDate;
        });
        dispatch(resultSetSelection(selectionNodes));
      }
    } else {
      dispatch(unSelect());
    }
  };

  if (!controller.current.getModel() || controller.current.getModel().length === 0) return null;
  return <TimelineTable controller={controller.current} onMouseUp={handleMouseUp} />;
};

const determineDateAttributes = (
  type: string,
  types: { [label: string]: { attributes: { [id: string]: { dimension: DimensionType } } } },
) => {
  const dateAttributes = Object.keys(types).reduce((acc, label) => {
    const attributes = types[label].attributes;
    let dateAttributes = Object.keys(attributes).filter((key) => TEMPORAL_ATTRIBUTES.includes(attributes[key].dimension));

    dateAttributes = dateAttributes.map((attribute) => {
      return type + '.' + label + '.' + attribute;
    });

    return [...acc, ...dateAttributes];
  }, [] as string[]);

  // console.log('USEMEMO dateAttributes', dateAttributes);
  return dateAttributes;
};

export interface CalendarVisProps {
  layout: string;
  showPopUpOnHover: boolean;
  shapes: {
    similar: boolean;
    shape: 'circle' | 'rectangle';
    shapeMap: { [id: string]: 'circle' | 'rectangle' } | undefined;
  };
  edges: {
    width: {
      similar: boolean;
      width: number;
    };
  };
}

export type CalendarProps = {
  showBarplot: boolean;
  itemsPerPage: number;
  displayAttributes: string[];
  displayEntity: string;
};

const configuration: CalendarProps = {
  itemsPerPage: 10,
  showBarplot: true,
  displayAttributes: [],
  displayEntity: '',
};

const CalendarSettings = ({
  configuration,
  graph,
  updateSettings,
}: {
  configuration: CalendarProps;
  graph: GraphMetadata;
  updateSettings: (val: any) => void;
}) => {
  useEffect(() => {
    if (graph && graph.nodes && graph.nodes.labels.length > 0) {
      updateSettings({ displayEntity: graph.nodes.labels[0] });
    }
  }, [graph]);

  const selectedNodeAttributes = useMemo(() => {
    if (configuration.displayEntity) {
      const nodeType = graph.nodes.types[configuration.displayEntity];
      if (nodeType && nodeType.attributes) {
        return Object.keys(nodeType.attributes);
      }
    }
    return [];
  }, [configuration.displayEntity, graph]);

  useEffect(() => {
    if (graph && graph.nodes && graph.nodes.labels.length > 0) {
      updateSettings({ displayAttributes: selectedNodeAttributes });
    }
  }, [selectedNodeAttributes]);

  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Select entity"
        value={configuration.displayEntity}
        onChange={(val) => updateSettings({ displayEntity: val })}
        options={graph.nodes.labels}
      />
      <Input
        type="boolean"
        label="Show barplot"
        value={configuration.showBarplot}
        onChange={(val) => updateSettings({ showBarplot: val })}
      />
      <Input
        type="dropdown"
        label="Items per page"
        value={configuration.itemsPerPage}
        onChange={(val) => updateSettings({ itemsPerPage: val })}
        options={[10, 25, 50, 100]}
      />
      <div>
        <span className="text-sm">Attributes to display</span>
        <div className="">
          <Input
            type="checkbox"
            value={configuration.displayAttributes}
            options={selectedNodeAttributes}
            onChange={(val: string[] | string) => {
              const updatedVal = Array.isArray(val) ? val : [val];
              updateSettings({ displayAttributes: updatedVal });
            }}
          />
        </div>
      </div>
    </SettingsContainer>
  );
};

export const CalendarComponent: VISComponentType = {
  displayName: 'CalendarVis',
  component: CalendarVis,
  settings: CalendarSettings,
  configuration: configuration,
};

export default CalendarComponent;
