import { GraphQueryResult } from '@graphpolaris/shared/lib/data-access/store';
import { DateValue } from './calendarvis.types';

import moment from 'moment';

/**
 * Calculates the ISO week number for a given date.
 *
 * @param {Date} date - The date for which to calculate the week number.
 * @returns {number} - The ISO week number. ISO weeks start on Monday and the first week of a year is the one that includes the first Thursday.
 *
 * @example
 * // Returns the ISO week number for December 31, 2021
 * getWeekNumber(new Date(2021, 11, 31));
 */
export function getWeekNumber(date: Date): number {
  const tempDate = new Date(date.getTime());
  tempDate.setHours(0, 0, 0, 0);
  tempDate.setDate(tempDate.getDate() + 3 - ((tempDate.getDay() + 6) % 7));
  const week1 = new Date(tempDate.getFullYear(), 0, 4);
  return 1 + Math.round(((tempDate.getTime() - week1.getTime()) / 86400000 - 3 + ((week1.getDay() + 6) % 7)) / 7);
}

/**
 * Calculates the week year for a given date.
 *
 * @param {Date} date - The date for which to calculate the week year.
 * @returns {number} - The week year.
 *
 * @example
 * // Returns the week year for December 31, 2021
 * getWeekYear(new Date(2021, 11, 31));
 */
export function getWeekYear(date: Date): number {
  const tempDate = new Date(date.getTime());
  tempDate.setDate(tempDate.getDate() + 3 - ((tempDate.getDay() + 6) % 7));
  return tempDate.getFullYear();
}

export function getWeeksInMonth(year: number, month: number): number[] {
  const weeks = new Set<number>();
  const startDate = new Date(year, month, 1);
  const endDate = new Date(year, month + 1, 0);

  for (let day = startDate; day <= endDate; day.setDate(day.getDate() + 1)) {
    weeks.add(getWeekNumber(new Date(day)));
  }

  return Array.from(weeks);
}

export function getFirstDayOfWeek(locale: string) {
  const baseDate = new Date(Date.UTC(2017, 0, 2)); // Just a Monday
  const weekDays = Array(7)
    .fill(1)
    .map((_, i) => baseDate.toLocaleDateString(locale, { weekday: 'short', timeZone: 'UTC' }));
  return weekDays;
}

export function getRandomDates(lengthDateArray: number): DateValue[] {
  const dateArray = [];

  const start = new Date(2022, 11, 1);
  const end = new Date();

  const delta = end.getTime() - start.getTime();

  for (let i = 0; i < lengthDateArray; i++) {
    const dayInBetween = Math.random() * delta;
    const randomDate = new Date(start.getTime() + dayInBetween);
    dateArray.push(randomDate);
  }

  const mean = (start.getTime() + end.getTime()) / 2;
  const standardDeviation = (end.getTime() - start.getTime()) / 6;

  const random = new SeededRandom(12345); // Use a constant seed for reproducibility

  const numbers = Array(lengthDateArray)
    .fill(0)
    .map(() => {
      // Use the Box-Muller transform to generate a normally distributed number
      const u1 = random.next();
      const u2 = random.next();
      const z0 = Math.sqrt(-2.0 * Math.log(u1)) * Math.cos(2.0 * Math.PI * u2);
      return z0 * standardDeviation + mean;
    });
  numbers.sort((a, b) => a - b);

  return dateArray.map((date, index) => {
    return {
      date,
      value: Math.floor(numbers[index]),
    };
  });
}

export function getDatesForAttribute(queryResult: GraphQueryResult, attributeName: string): DateValue[] {
  const split = attributeName.split('.');

  const queryResultNodes = queryResult['nodes'];
  // const queryResultEdges = queryResult['edges'];

  const dateArray: DateValue[] = [];
  if (split[0] === 'nodes') {
    queryResultNodes.forEach((node) => {
      if (node.label === split[1]) {
        const storedDate = node.attributes[split[2]];
        if (!storedDate) return;
        const dateObj = moment(storedDate).toDate();

        dateArray.push({
          date: dateObj,
          value: 1,
        });
      }
    });
  }
  return dateArray;
}





class SeededRandom {
  constructor(private seed: number) {}

  next() {
    this.seed = (this.seed * 16807) % 2147483647;
    return (this.seed - 1) / 2147483646;
  }
}

export function getContrastColor(hexColor: string): string {
  // If the color is not in hex format or doesn't have 6 digits, return default color
  if (hexColor.indexOf('#') !== 0 || hexColor.length !== 7) {
    return '#000000';
  }

  const r = parseInt(hexColor.slice(1, 3), 16);
  const g = parseInt(hexColor.slice(3, 5), 16);
  const b = parseInt(hexColor.slice(5, 7), 16);

  // Calculate the brightness of the color
  const brightness = (r * 299 + g * 587 + b * 114) / 1000;

  // If the color is bright, return black. If the color is dark, return white.
  return brightness > 155 ? '#000000' : '#FFFFFF';
}
